// Bruch.java                                       MM 2015

import Tools.Euklid.*;                            // Euklid

/**
 * Gemeine Brueche.
 */
public class Bruch
{
                                        // Instanzvariablen
/**
 * Zaehler.
 */
  private int zaehler;

/**
 * Nenner.
 */
  private int nenner;

/* ----------------------------------------------------- */
                                           // Konstruktoren
/**
 * Konstruktor,
 * wirft Ausnahme, falls Nenner == 0.
 * @param z Zaehler
 * @param n Nenner
 * @throws BruchException, NPlusException
 */
  public Bruch( int z, int n)
    throws Exception
  {
    if( n == 0) throw new BruchException();
    if( z == 0)
    {
      zaehler = 0;
      nenner = 1;
    }
    else
    {
      int i = Euklid.ggT( Math.abs( z), Math.abs( n));
      zaehler = z / i;
      nenner = n / i;
    }
  }

/**
 * Konstruktor, ganze Zahl als Bruch.
 * @param z ganze Zahl
 */
  public Bruch( int z)
  {
      zaehler = z;
      nenner = 1;
  }

/* ----------------------------------------------------- */
                                            // get-Methoden
/**
 * Lesen des Zaehlers.
 * @return Zahler des Bruches
 */
  public int getZaehler()
  {
    return zaehler;
  }

/**
 * Lesen des Nenners.
 * @return Nenner des Bruches
 */
  public int getNenner()
  {
    return nenner;
  }

/* ----------------------------------------------------- */
                                        // service-Methoden
/**
 * Setzen des Zaehlers.
 * @param z neuer Zaehler
 */
  public void setZaehler( int z)
  {
    zaehler = z;
  }

/**
 * Setzen des Nenners.
 * @param n neuer Nenner
 * @throws BruchException
 */
  public void setNenner( int n)
    throws Exception
  {
    if( n == 0) throw new BruchException();
    nenner = n;
  }

/* ----------------------------------------------------- */
                                             // Operationen
/**
 * Addition.
 * @param a Bruch, welcher addiert werden soll
 * @return Ergebnis der Addition
 * @throws BruchException, NPlusException
 */
  public Bruch add( Bruch a)
    throws Exception
  {
    int x = zaehler  * a.getNenner()
          + nenner * a.getZaehler();
    int y = nenner * a.getNenner();
    return new Bruch( x, y);
  }

/* ----------------------------------------------------- */
                                        // toString-Methode
/**
 * Darstellung eines Bruchs.
 * @return Bruch in linearer Schreibweise
 */
  public String toString()
  {
     return "" + zaehler + "/" + nenner;
  }

/* ----------------------------------------------------- */
/* ----------------------------------------------------- */
/**
 * Testprogramm zur Klasse Bruch,
 * 1: 2 / 3 + 3 / 18
 * 2: 3 * ( 225 / 375)
 * 3: 2 + 2 + 2 - 1
 */
  public static void main( String[] args)
  {
    try
    {
      Bruch a = new Bruch( 2, 3);                  // 5 / 6
      Bruch b = new Bruch( 3, 18);

      System.out.println
      ( "1: " + a + " + " + b + " = " + ( a.add( b)));

      Bruch c = new Bruch( 225, 375);              // 9 / 5

      System.out.println
      ( "2: 3 * " + c + " = " + c.add(( c.add( c))));

      Bruch d = new Bruch( 2);                     // 5 / 1
      Bruch e = new Bruch( -1);

      System.out.println
      ( "3: 3 * " + d + " + " + e + " = "
        + e.add( d.add( d.add( d))));
    }
    catch( Exception e)
    {
      System.out.println( "Fehler: " + e);
    }
  }
}
