// BruchAddiererController.java                     MM 2015
// MVC-Controller

import java.awt.event.*;                 // Listener, Event

/**
 * Controller zum BruchAddiererView.
 */
public class BruchAddiererController
  extends WindowAdapter
  implements ActionListener, KeyListener
{
/* ----------------------------------------------------- */
                                                     // MVC
/**
 * Mathematisches Modell,
 * enthaelt Funktionalitaet der Bruchaddition.
 */
  private BruchAddiererModel model;

/**
 * Zum Controller gehoeriger View.
 */
  private BruchAddiererView view;

/* ----------------------------------------------------- */
                                        // MVC-Installation
/**
 * Konstruktor, initialisiert Model und View.
 * @param model Mathematisches Modell
 * @param view zum Controller gehoeriger View
 */
  public BruchAddiererController
  ( BruchAddiererModel model, BruchAddiererView view)
  {
    this.model = model;
    this.view = view;
  }

/* ----------------------------------------------------- */
                                      // MVC-Deinstallation
/**
 * Freigabe des Controllers,
 * setzt Model und View zurck.
 */
  public void release()
  {
    model = null;
    view = null;
  }

/* ----------------------------------------------------- */
                             // handleEvent-Methoden Button
/**
 * ActionListener,
 * Ereignisverarbeitung Betaetigen eines Button.
 */
  public void actionPerformed( ActionEvent ae)
  {
    Object command = ae.getActionCommand();

                                      // setzt Startzustand
    if( command.equals( BruchAddiererView.ACTION_C))
    {
      model.reset();
      view.setFocusZaehler();
    }
                                     // fuehrt Addition aus
    if( command.equals( BruchAddiererView.ACTION_ADD))
    {
      try
      {
        model.addiere();
        view.setFocusZaehler();
      } catch( Exception e){}
    }
  }

/* ----------------------------------------------------- */
                           // handleEvent-Methoden Tastatur
/**
 * KeyListener,
 * Ereignisverarbeitung Druecken einer Taste,
 * Methode tut nichts.
 */
  public void keyPressed( KeyEvent ke){}


/**
 * KeyListener,
 * Ereignisverarbeitung Loslassen einer Taste,
 */
  public void keyReleased( KeyEvent ke)
  {
    Object quelle = ke.getSource();
    if( view.istZaehler( quelle))        // Eingabe Zaehler
    {
      try
      {                 // uebergibt Model eingegebene Zahl
        model.setOperandZaehler
        ( Integer.parseInt( view.getText( quelle)));
      }
      catch( NumberFormatException ne)
      {
        view.setFehlerDialog
        ( "FEHLER: " + view.getText( quelle)
                     + "\nGanze Zahlen eingeben!");
        view.resetZaehler();       // setzt Zaehler zurueck
      }
                             // schliesst Zaehlereingabe ab
      if( ke.getKeyChar() == '\n') view.setFocusNenner();
    }

    if( view.istNenner( quelle))          // Eingabe Nenner
    {
      try
      {                 // uebergibt Model eingegebene Zahl
        model.setOperandNenner
        ( Integer.parseInt( view.getText( quelle)));
      }
      catch( NumberFormatException ne)
      {
        view.setFehlerDialog
        ( "FEHLER: " + view.getText( quelle)
                     + "\nGanze Zahlen eingeben!");
        view.resetNenner();         // setzt Nenner zurueck
      }
      catch( Exception e)
      {
        view.setFehlerDialog( "" + e);
        view.resetNenner();         // setzt Nenner zurueck
      }
                              // schliesst Nennereingabe ab
      if( ke.getKeyChar() == '\n') view.setFocusAdd();
    }
  }

/**
 * KeyListener,
 * Ereignisverarbeitung Druecken und Loslassen
 * einer Taste, Methode tut nichts.
 */
  public void keyTyped( KeyEvent ke){}

/* ----------------------------------------------------- */
                            // handleEvent-Methoden Fenster
/**
 * WindowAdapter,
 * Ereignisverarbeitung Schliessen des Fensters,
 * Programm wird beendet.
 */
  public void windowClosing( WindowEvent we)
  {
    view.release();
  }
}